#!/usr/local/bin/perl
#┌─────────────────────────────────
#│ download.cgi - リンクから起動されてファイルをダウンロード
#｜ スガラボット 作成: 2008/02/14
#└─────────────────────────────────
#
# 2013/02/14 v1.53 ターゲットファイル名がcgiのGETリクエストのパラメータで指定されないときにHTTPエラーを出さないように修正
# 2011/10/15 v1.42 ダウンロードするファイルを読み込む時、メモリーに一括読み込みしていたのを 4MB のバッファを繰り返し使用するようにした
#

$sver = "v1.53";

use strict;
use warnings;
use lib './lib';
use Unicode::Japanese;
use File::Basename;

our $charcode = 'cp932';		# 日本語ファイル名はUTF-8で送られてくるため、必要なら大多数のPCコードに合わせる
our %in;
our $postflag;

# 2008/10/29 新規に v1.03として配布

#	our $charset = 'utf8';
#	our $logchar = 'utf8';
#	our $encode = 'utf8';
	require './init.cgi';

	&parse_form0;

	my %mime_type = ('jpg' => "image/jpeg", 'jpe' => "image/jpeg", 'gif' => "image/gif", 'png' => "image/png", 
		'txt' => "text/plain", 'pdf' => "application/pdf", 'doc' => "application/msword", 
		'dot' => "application/msword", 'xls' => "application/vnd.ms-excel",
		'xlt' => "application/vnd.ms-excel", 'ppt' => "application/vnd.ms-powerpoint",
		'pot' => "application/vnd.ms-powerpoint", 'zip' => "application/zip",
		'lzh' => "application/x-lzh", 'tar' => "application/x-tar", 'gz' => "application/x-tar",
		'bz' => "application/x-tar", 'cgi' => "text/plain", 'pl' => "text/plain", 'php' => "text/plain",
		'java' => "text/plain", 'c' => "text/plain", 'h' => "text/plain", 'html' => "text/html",
		'htm' => "text/html", 'shtml' => "text/html");

	our $roomdir = $in{'roomdir'};

#	our $upldir = "$roomdir/upl";
#	my $bfile = $upldir . '/' . $in{'bfile'};

	# 時間取得
	our ($time, $date) = &get_time;

	# ダウンロードするファイルが指定されていなければエラーで終了 (2013/02/14 スガラボット修正)
	if ( !defined($in{'bfile'}) ) { 
		my $ERROR;
		open( $ERROR, ">> ./data/download_error.log");
		print $ERROR "$date $time DOWNLOAD ERROR!! Target file is not specified. stopped.";
		close( $ERROR );
		&error( "Target file is not specified!! stopped." ); 
	}

	
	# プレビューでファイルのフルパスを渡すようにしたことによる修正 （2009/08/06 スガラボット修正)
	my $bfile;
	if ($in{'bfile'} =~ /\//) {
		$bfile = $in{'bfile'};
	} else {
#		$main::upldir = $roomdir . '/upl' if ($roomdir);
#		$bfile = $main::upldir . '/' . $in{'bfile'};
		our $upldir;
		if ( $roomdir ) { $bfile = "$roomdir/upl" . '/' . $in{'bfile'}; }
		else { $bfile = $upldir . '/' . $in{'bfile'}; }
	}

	my $fn = $in{'fn'};
	
#	my (undef, $ex) = split(/\./, $in{'bfile'});
	my $basenam = basename($bfile);
	my (undef, $ex) = split(/\./, $basenam);
	my $mime = $mime_type{$ex};

	open (BFILE,"$bfile") || &error("error: Server couldn't open $bfile"); 
	flock(BFILE,1); 
	binmode ( BFILE );
	binmode ( STDOUT );
	my ( $fsize ) = -s BFILE;

	my $buf;
	my $bsize = 4096 * 1024;			# set buffer size (4MB)

# .pdf をドコモ・スマートフォンで表示するため、拡張子が pdf ならMIMEタイプを正しく指定するための修正 （2012/06/14 スガラボット修正)
	my $content_type;
	if ($ex eq 'pdf') {
		$content_type = $mime;
	} else {
		$content_type = 'application/octet-stream';
	}

	my $head1 = "Content-Type: $content_type; name=\"$bfile\"\n"; 
	my $head2 = "Content-Disposition: attachment; filename=$fn\n"; 
	my $head3 = "Content-Length: $fsize\n\n";

	my $head = $head1 . $head2 . $head3;
#	print $head;

#	&error("bfile=$in{'bfile'}, extention=$ex, mime-type=$mime, header=<br>$head<br>\n");
	
	syswrite ( STDOUT, $head );

	my $remain = $fsize;
	my $length;
	while ( $remain > 0 ) {
		$length = read ( BFILE, $buf, $bsize );
		if ( !defined($length) ) { 
			my $ERROR;
			open( $ERROR, ">> ./data/download_error.log");
			print $ERROR "$date $time DOWNLOAD ERROR!! Specified file ($bfile) is not readable, stopped.";
			close( $ERROR );
			&error( "DOWNLOAD ERROR!! Specified file ($bfile) is not readable, stopped." ); 
		}
		syswrite ( STDOUT, $buf, $length );
		$remain -= $length;
	}

	close ( BFILE );

	# ダウンロードカウンタファイルに登録処理追加 (20080328 スガラボット)
#	&dlcount($fn);
	&dlcount($in{'fn_utf8'});
	
	exit;

#-------------------------------------------------
#  フォームデコード
#-------------------------------------------------
sub parse_form0 {

	my $buf;
	if ($ENV{'REQUEST_METHOD'} eq "POST") {
		$postflag = 1;
		read(STDIN, $buf, $ENV{'CONTENT_LENGTH'});
	} 
	else {
		$buf = $ENV{'QUERY_STRING'};
		# 余分な?がついていたら削除 (dowonload.cgi?room=xxxx&?なる対策)
		$buf =~ s/\?//;
		foreach ( split(/&/, $buf) ) {
			my($key, $val) = split(/=/);
			$val =~ tr/+/ /;
			$val =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("H2", $1)/eg;

			# UTF-8のファイル名も残しておく
			if ($key eq 'fn') {
				 my $tmp = $val;
				 $in{'fn_utf8'} = Unicode::Japanese->new($tmp, $main::charset)->getu;
				
			}
			# 日本語ファイル名がついていた場合はデコードする
#			if ($main::charset ne 'sjis') { $val = Unicode::Japanese->new($val, $main::charset)->conv($main::charcode); }
			$val = &strDecode($val, $main::charset);
			$val = &strEncode($val, $charcode);

			# ハッシュ化
			$in{$key} .= "\0" if (defined($in{$key}));
			$in{$key} .= $val;
		}
	}
}

#-------------------------------------------------
#  ダウンロードカウンタファイル登録処理
#-------------------------------------------------
sub dlcount {
	my ($fname) = @_;
	my ($fbase, $file, $total, $buf, $outbuf, $ID, $NUM);
	
	($fbase) = split(/\./, $fname);
	
	$file = "$main::upldir/dlcount.dat";

	my $DAT;
	if (-e $file) { 
		open($DAT, "+<$file") || &error("error: Server couldn't open $file");
		flock($DAT, 2);
		$total = <$DAT>;		
		chomp($total);
	} else {
		open($DAT, ">$file") || &error("error: Server couldn't create $file");
		flock($DAT, 2);
		$total = 0;
	}
	
	$total++;
	$outbuf = $total . "\n";
	
	my $i = 0;
	my $flag = 0;
	while ( $buf = &lineDecode($DAT) ) {
		$i++;
		chomp($buf);
		my ($id, $num) = split(/<>/, $buf);
		
		if ($id eq $fname) {
			$ID = $id;
			$num++;
			$NUM = $num;
			$flag = 1;
		}
		$outbuf .= $id . '<>' . $num . "\n";
	}
	
	if (!$flag && $fname) {
		$outbuf .= $fname . "<>1\n";
	}

	seek ($DAT, 0, 0);
#	$outbuf = Unicode::Japanese->new($outbuf, 'utf8')->utf8;
#	print $DAT $outbuf;
	print $DAT &strEncode($outbuf, $main::logchar);
	truncate($DAT, tell($DAT));
	flock($DAT, 8);
	close($DAT);
	my @val = ($ID, $NUM);
	return @val;
}

# 出力時のコード変換 (UTF8の時はUTFフラグを落とす)
sub strEncode {
	my ($str, $code) = @_;
	if (!$code) { $code = $main::charset; }
	$str = Unicode::Japanese->new($str, 'utf8')->conv($code);
	return $str;
}

# ファイル読み込み時のコード変換 (UTFフラグ付きの内部コードに変換)
sub strDecode {
	my ($str, $code) = @_;
	if (!$code) { $code = 'utf8'; }
	# 読み込んだ文字列をutf8内部コードに変換
	$str = Unicode::Japanese->new($str, $code)->getu;
	return $str;
}

# ファイル読み込み時のコード変換
sub lineDecode {
	my ($FH, $code) = @_;
	if (!$code) { $code = $main::logchar; }
	my $str = <$FH>;
	# 読み込んだ文字列をutf8内部コードに変換
	$str = Unicode::Japanese->new($str, $code)->getu;
	return $str;
}

1;

