#!/usr/local/bin/perl
# ---------------------------------------------------
# CGIファイル名: fileConvert.cgi
# 作成・著作権者: スガラボット
# ご利用条件: 個人的なご利用・及び再配布は自由です。但し商業利用されるときは下のホームURLのメールフォームから作者までご連絡ください。 
# ホームURL: http://vmorita.com/
# 掲示板URL: http://vmorita.com/bbs/patio/patio.shtml
# ---------------------------------------------------
#
# fileConvert.cgiは指定されたディレクトリまたはカレントディレクトリ配下のファイルの文字コードをShift_JIS(CP932)、utf-8、euc-jp と 7bit-jis の指定されたコード間で再帰的に一括変換します。
# ・このCGIツールはコマンドラインから実行すると、任意のディレクトリ/ファイルに対して処理出来ます。
# ・Forum Patio 掲示板のCGIディレクトリに置いて、ブラウザからアクセスすると、指定した掲示板会議室のログを一括変換することが出来ます。
# ・会議室ログファイルのコード変換のときは、①処理する会議室の選択、②文字コードの選択と処理するファイルを確認するプレビュー、③処理の実行 の3フェーズで処理します。
# ・コード変換処理実行後、「元に戻す」ボタンにより、直前に行った処理をそれ以前の状態に戻すことが出来ます。
#
# コマンドラインから実行するとき、コマンド形式は以下の通りです。
# ・perl fileConvert.cgi {-from=CODE1] [-to=CODE2] [-dir=DIR1] [-dir=DIR2] … [-dir=DIRN] [-ext=～] [-ng] [-r] [file1] [file2] … [fileN] [-undo] (リターン)
# ・"-from=CODE1" は変換元ファイルの文字コード指定、"-to=CODE2" は変換先の文字コードを指定します。 "CODE"としては "sjis"、"euc-jp"、"7bit-jis" と "utf8"が指定できます。
# ・"-from=CODE1" が省略されたときは "shiftjis"、"-to=CODE2" が省略されたときは "utf8" が指定されたものとして処理します。
# ・"-to=CODE2" は変換先ファイルの文字コード指定です。使用方法は "-from=CODE1" と同様です。
# ・"-" で始まるオプションは [file1] … [fileN] の前に置いてください。(後に置いても動作はします)
# ・[-dir=DIR1] [-dir=DIR2] … [-dir=DIRN] オプションは、"～"で処理対象のディレクトリ (絶対パスまたは相対パス) を指定します。複数のディレクトリが指定できます。
# ・"-dir=～"が省略されたときはカレントディレクトリが対象になります。
# ・"-ext=～"オプションは、"～"でターゲットファイルの拡張子(cg,log,datなど)を指定します。"-ext="のように"～"が無いときは全ファイルが対象となります。
# ・"-ext=～"オプションが指定されたときは、[file1] [file2] … [fileN] 形式のファイル指定は無視されます。
# ・デフォールトではこの処理は入力ファイルの文字列が shiftjis(CP932を含みます) 等指定のコードと判定できないときは変換を中止しますが、[-ng] があると強制変換します。
# ・"-r"オブションがあるときは、対象ディレクトリ配下の全ディレクトリを再帰的に処理します。
# ・[file1] [file2] … [fileN] で個別ファイルを指定するほか '*.cgi' や '??.cgi' のように"*"と"?"をワイルドカードとして使用できます。
# ・(注: ワイルドカードで指定するとき、LinuxなどUnix系のOSの時は必ず、上のように''で囲むか、\*.cgi のように「*」や「?」をエスケープしてください。Windowsでは必要ありません。)
# ・変換元のファイルは ターゲットディレクトリの下に新たに作成する "$CODE1_bak" というフォルダーにバックアップされます。 
# ・ファイルのコードを変換した後、前と全く同じオプションを指定して最後に "-undo" オプションを付けると前に行った変換をすべて元に戻す事が出来ます。
# ・全く何もパラメータが指定されていないときは、fileConvert.cgi は何も処理しません。
# ・処理内容は標準出力画面に出力されます。このとき、画面表示の文字コードはWindows/Linuxのコンソール文字コードに従って適切に変換します。

use strict;
use warnings;
use utf8;

my $termCharset;
if ($ENV{'WINDIR'}) { $termCharset = 'sjis'; }
else { (undef,$termCharset) = split(/\./, $ENV{'LANG'}); }

my $httpRequest = 0;
our $strong1 = '';
our $strong2 = '';
our $br = '';
my $end = '';
our ($room, $roomdir, %in, $def_room, $sub_dir, $root_dir, $bbs_dir);

if ($ENV{'REQUEST_METHOD'}) {
	$httpRequest = 1;
	$termCharset = 'utf-8';
	my $job = 'preview';
	$strong1 = '<font color=\"red\"><b>';
	$strong2 = '</b></font>';
	$br = '<br>';
	$end = '<br></body></html>';
}

my @targetDir = ();
our $extFlag = 0;
our $target = '';
our @targetFile = ();
our $recursive = 0;
our $noguess = 0;
our $undo = 0;
our $code1 = 'shiftjis';
our $code2 = 'utf8';
our $preview = 0;
if ($httpRequest) {
	print "Content-Type: text/html; charset=$termCharset\nPragma: no-cache\nCache-Control: no-cache\n\n";
}

use lib './lib';
use Unicode::Japanese;
use Unicode::Japanese qw(unijp PurePerl);

eval "use Encode;";
if ($@) { print "This server cannot execute this script. Please use perl version 5.8.1 or greater!!\n"; exit; }
#use Encode::Guess qw( euc-jp cp932 7bit-jis utf8 );

binmode STDOUT, ":encoding($termCharset)";
binmode STDERR,  ":encoding($termCharset)";

if ($httpRequest) {
#	print "Content-type: text/html\n\n";
	print <<"EOM";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html lang="ja">
<head>
<meta http-equiv="content-type" content="text/html; charset=$termCharset">
<title>会議室ログファイルの一括コード変換</title>
</head>
<body>
EOM
}

if ($httpRequest) {
#	nouse strict;
	require 'init.cgi';
	&parse_form;
	my $job = $in{'job'};
	print "<h3>$in{'room'}掲示板ログファイルの一括文字コード変換</h3>\n";
	if ($job eq 'convert') {
		$preview = 0;
		print "------------------- <b>以下のファイルの文字コード変換を実行します</b> --------------------<br>";
	} else {
		if ($def_room && !$in{'room'}) {
			# bbsルートディレクトリを読み込んでコード変換する会議室名を選択
			my $bbsdir;
			if ($sub_dir) {
				$bbsdir = "$root_dir/$sub_dir/$bbs_dir";	# 掲示板をサブディレクトリに置くとき
			} else {
				$bbsdir = "$root_dir/$bbs_dir";	# 掲示板をサブディレクトリに置くとき
			}
			my $bbsFH;
			unless (opendir($bbsFH, $bbsdir)) { print "${bbsdir} の情報を取得できません$end\n"; exit; }
			my (@file) = readdir($bbsFH);
			closedir($bbsFH);

			print "<form action=\"fileConvert.cgi\" method=\"GET\">ログファイルのコード変換を行う会議室をチェックしてください:<br> \n";

			my $f;
			foreach $f (@file) {
				if ($f eq '.' || $f eq '..') { next; }
				my $FH;
				if (opendir($FH, "$bbsdir/$f")) {
#					print "file=$f<br>\n";
					my $checked = '';
					if ($f eq $def_room) { $checked = 'checked'; }
					print "<input type=\"radio\" name=\"room\" value=\"$f\" $checked> $f \n";
					closedir($FH);
				} else { next: }
			}
			
			print "<br><br><input type=\"submit\" name\"run\" value=\"選択して送信\"></form>$end\n";
			exit;
		} else {
			# 会議室名が選択されていればコード変換前のプレビュー
			$preview = 1;
			print <<"EOM";
<form action="fileConvert.cgi" method="GET"><br>
現在の文字コード:&nbsp;&nbsp; <input type="radio" name="from" value="sjis" checked> sjis 
<input type="radio" name="from" value="utf8"> utf8 
<input type="radio" name="from" value="euc-jp"> euc-jp  
<br>変換後の文字コード: <input type="radio" name="to" value="sjis"> sjis 
<input type="radio" name="to" value="utf8" checked> utf8 
<input type="radio" name="to" value="euc-jp"> euc-jp  
<br><br>
------------------- <b>[プレビュー] 以下のファイルが文字コード変換の対象となります</b> --------------------<br>
EOM
		}
	}
#	@targetDir = split(/\0/, $in{'dir'});
	if (!$roomdir) { $roomdir = '.'; }
	@targetDir = ("$roomdir/data", "$roomdir/log", "$roomdir/delsave");
	$code1 = $in{'from'};
	$code2 = $in{'to'};
	$recursive = 1;
	$undo = $in{'undo'};
	@targetFile = ('*.log', '*.cgi');
} else {
	foreach (@ARGV) {
	#	print $_ . "\n";
		if ($_ =~ /^-dir=(.+)/) { push (@targetDir, $1); next; }
		if ($_ =~ /^-ext=(\w*)/) { $extFlag = 1; $target = $1; next; }
		if ($_ =~ /^-from=([\w\-]+)/) { $code1 = &codeCheck($1) || die("文字コードの指定が違います${br}\n"); next; }
		if ($_ =~ /^-to=([\w\-]+)/) { $code2 = &codeCheck($1) || die("文字コードの指定が違います${br}\n"); next; }
		if ($_ eq '-ng') { $noguess = 1; next; }
		if ($_ eq '-r') { $recursive = 1; next; }
		if ($_ eq '-undo') { $undo = 1; next; }
		push (@targetFile, decode($termCharset, $_));
	}
}

our $backDir = "${code1}_bak";

print "処理パラメータ: room=$room, extFlag=$extFlag, target=$target, code1=$code1, code2=$code2, noGuess=$noguess, recursive=$recursive${br}\n";

my $dot;
if (@targetDir) {
	print "ターゲットディレクトリ:  ";
	$dot = '';
	foreach (@targetDir) {
		print $dot . $_;
		$dot = ', ';
	}
	print " ${br}\n";
}

print "ターゲットの拡張子: " . $target . "  ${br}\n" if ($extFlag);
if (@targetFile) {
	print "ターゲットファイル: ";
	$dot = '';
	foreach (@targetFile) {
		print $dot . $_;
		$dot = ', ';
	}
	print "${br}${br}\n\n";
}

my $pre = '';

if (!@targetDir) { push(@targetDir, '.'); }

foreach my $dirName (@targetDir) {
	&dirConvert($dirName, $pre);
}

print "######### ファイルの文字コード変換処理を終了しました #########$end\n";

if ($httpRequest) {
	if ($preview) {
		print <<"EOM";
<br><input type="hidden" name="job" value="convert">
<input type="hidden" name="room" value="$room">\n
<input type="submit" name="run" value="コード変換を実行する"> &nbsp;&nbsp;
<input type="button" value="前の画面に戻る" onClick="history.go(-1)">
</form>
EOM
	} else {
		print <<"EOM";
<br><form action="fileConvert.cgi" method="GET">\n
<input type="hidden" name="room" value="$room">\n
<input type="hidden" name="job" value="convert">\n
<input type="hidden" name="from" value="$code1">\n
<input type="hidden" name="to" value="$code2">\n
<input type="hidden" name="undo" value="1">\n
<input type="submit" name="run" value="前の変換処理を元に戻す"><br><br>\n
</form>
<form action="fileConvert.cgi" method="GET">\n
<input type="hidden" name="room" value="$room">\n
<input type="hidden" name="job" value="preview">\n
<input type="submit" value="もう一度プレビュー画面で確認する"> &nbsp;&nbsp;\n
<input type="button" value="二つ前の画面に戻る" onClick="history.go(-2)">\n
</form>
EOM
	}
}
exit;

sub dirConvert {

	my ($dirName, $pre, $DIR) = @_;
	my $code1here = ($code1 eq 'euc-jp') ? 'euc' : $code1;
	my $code2here = ($code2 eq 'euc-jp') ? 'euc' : $code2;
	my $currentDir = $dirName;
	if (!$currentDir) { $currentDir = '.'; }

	print "${pre}********* $currentDir ディレクトリの文字コード変換を開始します *********${br}\n";

	unless (opendir($DIR, "$currentDir")) { print "${pre}${currentDir} の情報を取得できません\n"; return; }
	my (@file) = readdir($DIR);
	closedir($DIR);

	my $f;
	my $count = 0;
	foreach $f (@file) {
		my $f_enc = decode($termCharset, $f);
		print "${pre}file= " . $f_enc . " ";
		my ($fb, $ext) = split(/\./, $f_enc, 2);
		if ($f eq '.' || $f eq '..') { print "スキップ${br}\n"; next; }
		my $nextDir = $dirName ? "$dirName/$f" : $f;
		my $FH;
		if (opendir($FH, "$nextDir")) { 
			print "これはディレクトリです${br}\n";
			if ($f eq "${code1}_bak") { print "バックファイルに付きスキップ${br}\n"; next; }
			if ($recursive) {
				&dirConvert($nextDir, "${pre}　　", $FH);
				next;
			}
			else { print "スキップ${br}\n"; next; }
		}
		if ($f_enc  =~ /^fileConvert\w*\.cgi/) { print "スキップ${br}\n"; next; }
		if ($fb =~ /_${code1}_bak$/) { print "スキップ${br}\n"; next; }
		if ($extFlag) {
#			print "extFlag=$extFlag, file=$f \n";
			if ($target && $ext ne $target) { print "スキップ${br}\n"; next; }
		} else {
			my $ok = 0;
			my $tfile;
			foreach $tfile (@targetFile) {
				if ($f_enc eq $tfile) { $ok = 1; last; }
				my $temp = $tfile;
				$temp =~ s/\*/\\S*/;
				$temp =~ s/\?/\\S/;
				$temp =~ s/\./\\./;
#				print "${pre}tfile=$temp\n";
				if ($f_enc =~ /^$temp$/) { $ok = 1; last; }
				next;
			}
			if (!$ok) { print "スキップ${br}\n"; next; }
		}

		# バックファイルがあるとき、undo処理ならバックファイルを元のファイルにリネーム、変換処理ならスキップ
		if (-e "${currentDir}/${backDir}/${f}") {
			if ($undo) {
				rename("${currentDir}/${backDir}/${f}", "${currentDir}/${f}");
				print "${strong1}ファイルのコード変換を元に戻しました${strong2}${br}\n";
				$count++;
				next;
			} else {
				print "同処理のバックファイルがあるためスキップ${br}\n";
				next;
			}
		}

		# もし undo 処理なら以降をスキップ
		if ($undo) { print "スキップ${br}\n"; next; }

		# もしバックアップディレクトリがなければ作成する
		unless (-e "${currentDir}/${backDir}") { mkdir("${currentDir}/${backDir}", 0777); }

		# もし preview 処理なら処理対象ファイルの変換をプリントして以降をスキップ
		if ($preview) {
			print "${strong1}の文字コードを変換します${strong2}${br}\n";
			$count++;
			next;
		}
		
		rename("${currentDir}/${f}", "${currentDir}/${backDir}/${f}");
		my ($IN, $OUT);
#		print "${pre}これから ${currentDir}/${backDir}/${f} ファイルを読み込みます\n";
		unless (open $IN, '<', "${currentDir}/${backDir}/${f}") { print "のバックアップファイルが読み込めませんでした${br}\n"; next; }

		# パーミッション取得
		my $permission = (stat($IN))[2] % 512;

		# データ読み込み
		my $data;
		$data .= $_ while (<$IN>);

		my $inst = Unicode::Japanese->new;

		# 入力文字コードを判定 (指定コード以外のファイルは変換しない)
		my $encName = $inst->getcode($data);
		if (!$noguess && $encName ne $code1here) {
			close($IN);
			rename("${currentDir}/${backDir}/${f}", "${currentDir}/${f}");
			print "${strong1}の文字コードは \"$encName\" で \'$code1here\' ではないため変換を中止しました${strong2}${br}\n";
			next;
		}

		unless (open $OUT, '>', "${currentDir}/${f}") {
			rename("${currentDir}/${backDir}/${f}", "${currentDir}/${f}");
			print "のファイルがオープンできませんでした${br}\n";
			next;
		}

#		print "の文字コード \"は$encName\" です。";
		$inst =Unicode::Japanese->new($data, $code1here);
		print $OUT $inst->$code2here;

		print "${strong1}の文字コードを変換しました${strong2}${br}\n";
		close($OUT); 
		close($IN);
		chmod($permission, "${currentDir}/${f}");
		$count++;
	}
	if ($count) {
		$pre = $pre . $strong1;
		$br = $strong2 . $br;
	}
	if ($undo) {
		print "${pre}********* $dirName ディレクトリでは $count 個のファイルを元に戻しました *********${br}\n";
	} elsif ($preview) {
		print "${pre}********* $dirName ディレクトリでは $count 個のファイルを変換します *********${br}\n";
	} else {
		print "${pre}********* $dirName ディレクトリでは $count 個のファイルを変換しました *********${br}\n";
	}
}

sub codeCheck {
	($_) = @_;
	if (/shift.*jis/i || /sjis/i) { return 'sjis'; }
	elsif (/euc.*jp/i || /jp.*euc/i || /ujis/i) { return 'euc-jp'; }
	elsif (/jis/i) { return '7bit-jis'; }
	elsif (/utf-?8/i) { return 'utf8'; }
	else { return ''; }
}
