# かばオリジナルperlライブラリ
#    Ver.1.00 (00.12.29) uniqueのバグ修正
#    Ver.1.01 (01.01.29) &getcookieのLOG出力をやめた。
#    Ver.1.02 (01.03.17) getweek、lockopen、lockclose追加。
#    Ver.1.03 (01.11.12) &reversetimeの閏年処理のバグ修正。
#    Ver.1.10 (02.10.29) &getJholiday追加。
#    Ver.1.11 (02.12.30) ファイルロックを改良
#    Ver.1.20 (03.01.28) &reversetimeの閏年処理の重大なバグを修正。
#---------------------------------------------------------------------------------------
#   【サブルーチン一覧】
#    weberror     HTML形式でエラーメッセージを出す
#    error        メッセージをprintしてexitする
#    warning      "Warning! "とメッセージをprintするだけ
#    reversetime  (年、月、日、[時、分、秒])をTIME形式にする
#    correcttime  TIME形式を(年、月、日、曜日(漢字)、時、分、秒)で返す
#    correcttime_nw  TIME形式を(年、月、日、曜日(0-6)、時、分、秒)で返す
#    deltadays    (年A、月A、日A、年B、月B、日B)でA-Bの日数を返す
#    caldays      (年、月、日、±n)で日付からnを加算した日付を(年、月、日)で返す
#    getcookie    クッキーを読んでデコードする
#    cookielimit  (年、月、日、期限までの日数)をクッキーの形式にして返す
#    getDate      現在日を(年、月、日、曜日0-6)で返す
#    unique       リスト中の文字列をユニーク化する
#    getweek      (年、月、日)を入れると曜日0-6を返す
#    getJholiday  (年、月、日)を入れると曜日0-6と祝日･平日などの属性を返す
#    lockopen     ロック付きでファイルをオープンする
#    lockclose    lockopenで開いたファイルを閉じる
#---------------------------------------------------------------------------------------

#ブラウザにエラーメッセージを表示するサブルーチン
#    引数： メッセージ、[ログのファイルハンドラ、ログファイル名]

use utf8;

sub weberror {
    my( $msg, $fh, $fn ) = @_ ;
    $msg =~ s/\n/<BR>\n/g;

    print "Content-Type: text/html\n\n";
    print "<HTML>\n";
    print "<HEAD>\n";
    print "  <META http-equiv=\"Content-Type\" content=\"text/html; charset=shift_jis\">\n";
    print "  <META name=\"Author\" content=\"Sato\">\n";
    print "  <TITLE>Error!</TITLE>\n";
    print "</HEAD>\n";
    print "<BODY>\n";
    print &strEncode("$msg\n");
    print "</BODY>\n";
    print "</HTML>\n";

    if (( $fh ne '' ) and ( $fn ne '' )) {
        &lockclose( $fh, "$fn" );
    }
    exit(0);
}

#エラー用サブルーチン。
sub error {
    print "Error!  ";    #とにかくエラーだって言わないとね。
    print &strEncode(@_ );           #デフォルト引数にあるメッセージを表示。
    exit;
}

#ワーニング用サブルーチン。
sub warning {
    print "Warinig!  ";  #ワーニングを宣言。
    print &strEncode(@_ );           #デフォルト引数にあるメッセージを表示。
}

#1970年1月1日 0:00:00 グリニッジ標準時から引数の時刻(日本時間)までの秒数を計算する
#    引数： 年、月、日、[時、分、秒]
#    戻り値： 970年1月1日 0:00:00 GMT からの秒数
#    01.11.12 閏年処理のバグ修正
sub reversetime{

    #ローカル変数の宣言
    my( $lyear, $lmon, $lday, $lhour, $lmin, $lsec ) = @_ ;
    my( $dyear, $uruudoshi, $yearsday, $monsday, @monlist, $amari );
    my( $result );

    #時分秒が指定されてなかったらデフォルト値を入れよう
    $lhour = 0  if !$lhour ;
    $lmin = 0  if !$lmin;
    $lsec = 0  if !$lsec;

    #年月日が無かったらエラーとして０を返す
    return 0 if !$lday;
    return 0 if !$lmon;
    return 0 if !$lyear;

    #指定された年から1970を引いて、閏年をカウントする。（当年こみ）
    $dyear = $lyear - 1970 ;
    $uruudoshi = ( $lyear - 1968 ) / 4;
    $uruudoshi = int($uruudoshi);
    $amari = ( $lyear - 1968 ) % 4;

    #閏年を考慮して年を日に換算する。
    #ただし$amariが0だと、その年が閏年なので、２月までは閏年１つ分カウントしない。
    #Ver.1.20で重大なバグを修正( $lmon > 3 )になっていた。
    if (( $amari == 0 ) and ( $lmon < 3 )) { $yearsday = $dyear * 365 + ($uruudoshi-1) }
    else { $yearsday = $dyear * 365 + $uruudoshi }

    #月によって日数が違うことを考慮して月を年初からの日数に換算する
    @monlist = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    $monsday = 0 ;
    for( $int = 0; $int < $lmon-1 ; $int++ ){
        $monsday = $monsday + $monlist[$int] ;
    }

    #年月日と時刻を日本時間のまま秒に換算する
    $result = ( $yearsday + $monsday + ($lday-1)) *86400 + $lhour*60*60 + $lmin*60 + $lsec ;
    
    #日本時間よりグリニッジ標準時の方が９時間遅いのでその分引き算
    $result = $result - 9*60*60 ;

    #さて結果を返そう
    $result;
}

#localtimeを実行してその戻り値を年・月・日・曜日・時・分・秒の配列で返す
#    引数：1970年1月1日 0:00:00 グリニッジ標準時からの秒数(reversetimeの結果)
#    戻り値：年、月、日、曜日(漢字)、時、分、秒
sub correcttime {

    my( $argsec ) = $_[0] ;
    #print "correcttimeが受け取った秒数は $argsec\n" ;
    my( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime( $argsec );

    $mon++;
    $year = $year+1900 ;
    my(@week) = ( '日','月','火','水','木','金','土' );
    my($jwday) = $week[$wday] ;

    #print "correcttimeの中では $year, $mon, $mday, $jwday, $hour, $min, $sec\n" ;
    ( $year, $mon, $mday, $jwday, $hour, $min, $sec );
}

sub correcttime_nw {

    my( $argsec ) = $_[0] ;
    #print "correcttimeが受け取った秒数は $argsec\n" ;
    my( $sec, $min, $hour, $mday, $mon, $year, $wday ) = localtime( $argsec );

    $mon++;
    $year = $year+1900 ;

    #print "correcttimeの中では $year, $mon, $mday, $jwday, $hour, $min, $sec\n" ;
    ( $year, $mon, $mday, $wday, $hour, $min, $sec );
}

#２つの日付の差の日数を計算するＡ－Ｂ
#    引数： 年A、月A、日A、年B、月B、日B
#    戻り値： 差の日数
sub deltadays {

    my( $yearA, $monthA, $dayA, $yearB, $monthB, $dayB ) = @_ ;
    my( $timeA, $timeB, $delta, $result, $daysec );

    $timeA = &reversetime( $yearA, $monthA, $dayA );
    $timeB = &reversetime( $yearB, $monthB, $dayB );
    $delta = $timeA -$timeB ;
    $daysec = 60*60*24;

    $result = $delta / $daysec;

    $result;
}

#日付の加算・減算
#    引数： 年、月、日、加算または減算したい日数
#    戻り値： 加算または減算した結果の年、月、日
sub caldays {

    my( $year, $month, $day, $sint ) = @_ ;
    my( $time, $daysecs, $ryear, $rmonth, $rday );

    $time = &reversetime( $year, $month, $day );
    $daysecs = $sint*60*60*24;

    $time = $time + $daysecs ;

    ( $ryear, $rmonth, $rday ) = &correcttime( $time );

    ( $ryear, $rmonth, $rday );
}

#クッキーを読んでデコードする
#    引数： 環境変数HTTP_COOCKIEをハッシュに入れる
#    戻り値： ハッシュを返す
#
sub getcookie
{
	my($cookie) = $ENV{'HTTP_COOKIE'};
	local(*cookies);
	my(@list, $name, $val);

	@list = split(/;\s*/, $cookie);

	foreach (@list) {
		($name, $val) = split(/=/, $_, 2);
		$val =~ tr/+/ /;
        $val =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("C", hex($1))/eg;
		$val = Unicode::Japanese->new($val, $encode)->getu;
		$cookies{$name} = $val;
	}

	return *cookies;
}

#クッキーのリミットを計算し整形した文字列にして返す
#    引数： 年、月、日、期限までの日数
#    戻り値： クッキー用に整形した文字列(eq. Fri, 31-Dec-1999 23:59:59 GMT )
sub cookielimit
{
    my( $ayear, $amon, $aday, $limit ) = @_ ;

    my(@week) = ( 'Sun','Mon','Tue','Wed','Thu','Fri','Sut' );
    my(@month) = ( 'Jun','Feb','Mar','Apl','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec' );

    my($time) = &reversetime( $ayear, $amon, $aday );
    $limit = $limit*60*60*24;
    $time = $time + $limit ;
    my( $sec, $min, $hour, $mday, $mon, $year, $wday ) = gmtime( $time );

    $year = $year+1900 ;
    my($ewday) = $week[$wday] ;
    my($emon) = $month[$mon] ;

    my($str) = $ewday. ', '. $mday. '-'. $emon. '-'. $year. ' 00:00:00 GMT' ;
    return($str);
}

# 現在時刻を入手する
#     引数：  なし
#     戻り値： 日時(String)
sub getDate{

    my($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime(time);
    $year += 1900;
    $mon++;

    return $year, $mon, $mday, $wday;
}

#１つの配列に同じ要素があったら削る
#    引数： 配列
#    戻り値： ユニークにした配列
sub unique
{
    my( @list1 ) = @_ ;
    my( @list2, $cell1, $cell2, $flag );

    foreach $cell1 ( @list1 ) {
        $flag = 0;
        foreach $cell2 ( @list2 ) {
            if ( $cell1 eq $cell2 ) { 
                $flag = 1;
                last;
            }
        }
        if ( $flag == 0 ) { push( @list2, $cell1 ) }
    }
    @list2;
}

# 曜日を返す
#     引数：年、月、日
#     戻り値：曜日(0～6、0は日曜、1は月曜...6は土曜)
sub getweek
{
    my( $year, $month, $day ) = @_ ;
    my( $week );

    #１月と２月は特別処理
    if ( $month <= 2 ) {
        $year--;
        $month = $month + 12 ;
    }

    $week = (($year+int($year/4)-int($year/100)+int($year/400)+int((13*$month+8)/5)+$day)%7);
    return $week;
}

# 曜日と祝日かどうかを返す
#     引数：年、月、日
#     戻り値：曜日(0～6、0は日曜、1は月曜...6は土曜)、平日0/祝日1、何の日
#     ※平成15年1月1日発行のハッピーマンデー法第２段対応。
#     平成14年までのカレンダーとは「海の日」と「敬老の日」が合いません。
#     さらに、春分と秋分は天文学上の日であって閣議決定された祝日とは違うかも知れません。
#     しかも簡易計算方法で、1980以降かつ2099以前のみ有効。
sub getJholiday
{
    my( $year, $month, $day ) = @_ ;
    my( $week, $holy, $str, $int, $remain, $x );

    $week = &getweek( $year, $month, $day );

    #祝日を識別
    $holy = 0; $str = '平日';
    if ( $month == 1 ) {
        if ( $day == 1 ) { $holy = 1; $str = '元日'; }
        else {
            $int = int( $day / 7 );
            $remain = $day % 7;
            if ( $remain > 0 ) { $int++; }
            if (( $int == 2 ) and ( $week == 1 )) { $holy = 1; $str = '成人の日'; }
        }
    }
    elsif ( $month == 2 ) {
        if ( $day == 11 ) { $holy = 1; $str = '建国記念日'; }
        elsif (( $day == 12 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
    }
    elsif ( $month == 3 ) {
        $x = int(20.8431+0.242194*($year-1980)-int(($year-1980)/4));
        if ( $day == $x ) { $holy = 1; $str = '春分の日'; }
        elsif (($day == ($x++)) and ($week==1) ) { $holy = 1; $str = '振替休日'; }
    }
    elsif ( $month == 4 ) {
        if ( $day == 29 ) { $holy = 1; $str = 'みどりの日'; }
        elsif (( $day == 30 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
    }
    elsif ( $month == 5 ) {
        if    ( $day == 3 )  { $holy = 1; $str = '憲法記念日'; }
        elsif ( $day == 4 )  { $holy = 1; $str = '国民の休日'; }
        elsif ( $day == 5 )  { $holy = 1; $str = 'こどもの日'; }
        elsif (( $day == 6 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
    }
    elsif ( $month == 7 ) {
        $int = int( $day / 7 );
        $remain = $day % 7;
        if ( $remain > 0 ) { $int++; }
        if (( $int == 3 ) and ( $week == 1 )) { $holy = 1; $str = '海の日'; }
    }
    elsif ( $month == 9 ) {
        $int = int( $day / 7 );
        $remain = $day % 7;
        if ( $remain > 0 ) { $int++; }
        if (( $int == 3 ) and ( $week == 1 )) { $holy = 1; $str = '敬老の日'; }
        else {
            $x = int(23.2488+0.242194*($year-1980)-int(($year-1980)/4));
            if ( $day == $x ) { $holy = 1; $str = '秋分の日'; }
            elsif (($day == ($x++)) and ($week==1) ) { $holy = 1; $str = '振替休日'; }
        }
    }
    elsif ( $month == 10 ) {
        $int = int( $day / 7 );
        $remain = $day % 7;
        if ( $remain > 0 ) { $int++; }
        if (( $int == 2 ) and ( $week == 1 )) { $holy = 1; $str = '体育の日'; }
    }
    elsif ( $month == 11 ) {
        if ( $day == 3 ) { $holy = 1; $str = '文化の日'; }
        elsif (( $day == 4 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
        elsif ( $day == 23 ) { $holy = 1; $str = '勤労感謝の日'; }
        elsif (( $day == 24 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
    }
    elsif ( $month == 12 ) {
        if ( $day == 23 ) { $holy = 1; $str = '天皇誕生日'; }
        elsif (( $day == 24 ) and ( $week == 1 )) { $holy = 1; $str = '振替休日'; }
    }

    return ( $week, $holy, $str );
}

# ファイルにロックをかけた上でオープンする
#     引数：fh(ハンドル)、modefile(モード付きファイル名)
#     戻り値：true or false
#
sub lockopen(*$)
{
    my($fh, $modefile) = @_;
    my($lockf, $path, $filename, $i, $flag);
    my($retry) = 5;                # リトライ回数

    my($mode, $file) = ($modefile =~ /^(\+?(?:<|>>?)\s*?)(.+)$/);
    if ($file =~ /(\/|\\)/) {
        ($path, $filename) = ($file =~ /^(.*[\/|\\])(.+)$/);
    }
    else {
        $path = '';
        $filename = $file;
    }

    if ( $filename eq '' ) { return undef }
    $lockf = $path.'Lock_'."$filename";

    #mkdirでロック
    $flag=0;
    for ( $i=0; $i<$retry; $i++ ) {
	if (mkdir("$lockf", 0755)) {
            $flag=1; last;
        }
        elsif ( $i == 0 ) {
            my( $makedTime ) = (stat($lockf))[9];
            if ($makedTime < time() - 600) {
                rmdir($lockf);
            }
        }
        else {
            sleep(1);
        }
    }

    #ロックが成功していたらファイルをオープン
    if ( $flag ) {
        if ( open($fh, $modefile) != 1 ) {
            close($filename);
 	    rmdir($lockf);
        }
        return 1;
    }
    else {
        return undef;
    }
}

# ロックファイルを消してファイルをクローズする
#     引数：fh(ハンドル)、file(ファイル名)
#     戻り値：なし
#
sub lockclose(*$)
{
    my($fh, $file) = @_;
    my($path, $filename, $lockf);

    if ($file =~ /(\/|\\)/) {
        ($path, $filename) = ($file =~ /^(.*[\/|\\])(.+)$/);
    }
    else {
        $path = '';
        $filename = $file;
    }
    $lockf = $path.'Lock_'."$filename";
    close($filename);
    rmdir($lockf);
    close($fh);
}

1;
