#!/usr/local/bin/perl
#-----------------------------------------------------------------------------
# File: 	reconf_plog.cgi
# Auther: 	Sugarabotto
# Date: 	2010/12/06
# Summary: 	Frorum Patio 掲示板の過去ログアーカイブを再構築する CLI スクリプト
#-----------------------------------------------------------------------------
#
#use strict;
#use warnings;
use utf8;

my $time0 = time;

# 外部ファイル取り込み
require './init.cgi';

my ($termCharset, $htmlCharset);
if ($ENV{'WINDIR'}) { $termCharset = 'sjis'; }
else { (undef,$termCharset) = split(/\./, $ENV{'LANG'}); $termCharset = 'utf8'; }

# エンコード種別
our $charset = $termCharset;
our $logchar = 'utf8';

our (%in, $bbs_dir, $bbs_url);

use lib './lib';
use Unicode::Japanese;
use Unicode::Japanese qw(unijp PurePerl);

eval "use Encode;";
if ($@) { 
	print "Content-Type: text/html\n\n";
	print "This server cannot execute this script. Please use perl version 5.8.1 or greater!!\n"; exit; 
}
#use Encode::Guess qw( euc-jp cp932 7bit-jis utf8 );

binmode STDOUT, ":encoding($termCharset)";
binmode STDERR, ":encoding($termCharset)";

$bbs_url = $ARGV[0];
print "--------------------- 過去ログアーカイブの再構築を実行します ----------------------\n";

&reconf_pastlog;

my $time1 = time;
my $exe_time = $time1 - $time0;
print "---------- 過去ログアーカイブの再構築処理を終了します (所要時間: ${exe_time}秒) -----------\n";

exit;

#-------------------------------------------------
#  過去ログ・再構築
#-------------------------------------------------
sub reconf_pastlog {

	my ($exit_flag) = @_;

	# 現行ログファイルオープン
	my ($INDEX1, $INDEX2);
	my (@tp, @tp2, @past_tp, $top, $past_tp_ln, @new_tp, @new_tp2, @past_newtp);

	# 現行インデックスファイルオープン
	open($INDEX1,"+< $nowfile") || &error("Open Error: $nowfile");
	@tp = &arrayDecode($INDEX1);
	$top = shift(@tp);
	push(@new_tp, $top);

	# 現行ログのスレッド数カウント
	my $now_count = @tp;

	# 過去ログインデックスファイルオープン
	# 現行インデックスファイルオープン
	open($INDEX2,"+< $pastfile") || &error("Open Error: $pastfile");
	@tp2 = &arrayDecode($INDEX2);
	my $tp2_count = @tp2;

	# 現行ログのスレッド数が $i_max より大きいか、再構築時、過去ログから現行ログに自動復帰なら現行ログインデックスと過去ログインデックスを連結する
	my $nowfile_change;
	if (($move2plog && $now_count > $i_max) || ($plog_restore && $tp2_count && $now_count < $i_max)) {

print "現行ログと過去ログをバックアップします。\n";

		# 重要なログファイルは変更する前にバックアップをとっておく。
		use File::Copy;
		copy("$nowfile", "$nowfile.bak");	
		copy("$pastfile", "$pastfile.bak");	

		# 現行ログに過去ログを連結
		push(@tp, @tp2);
		$nowfile_change = 1;
	} else { 
		push(@new_tp, @tp);
		@tp = @tp2;
	}
	my $tp_count = @tp;

	# 過去ログ・現行ログをロックする
	eval "flock($INDEX1, 2);";
	eval "flock($INDEX2, 2);";

	local $n = "1";
	local $acount = 0;
	local $N;
	local ($INDEX_P, @past_index, @past_page_dir);

	if (($nowfile_change && $tp_count > $i_max) || (!$nowfile_change && $tp2_count)) { 

		# 過去ログ書庫ページディレクトリがあれば一旦全削除する
		if (-e $pastdir) {

			use File::Path;
			my @pathArray = ("$pastdir");

			# ディレクトリを再帰的に全削除
			rmtree(@pathArray);

print "既存の過去ログアーカイブを一旦削除しました。\n";

			# すぐ復帰すると次のディレクトリ再作成を失敗するようなので一休み
			sleep(1);
		}

		# 過去ログ書庫ページディレクトリ作成
		mkdir($pastdir, '0707');

		# 過去ログ書庫インデックス・ファイル名の定義
		$N = sprintf("%04d", $n);
		$past_index[$n] = "$pastdir/P${N}.dat";

print "過去ログインデックスファイル: $past_index[$n]を作成しました。\n";

		open($INDEX_P, "+> $past_index[$n]");

		# 過去ログ書庫ファイルディレクトリの作成
		if ($copy_logfiles) {
			$past_page_dir[$n] = "$pastdir/plog${N}";
			unless (-e $past_page_dir[$n]) {
				mkdir($past_page_dir[$n], '0707');
				mkdir("$past_page_dir[$n]/upl", '0707');
				$acount++;
			}
		}

		# 過去ログ書庫ページのカウントファイルを作成
		my $COUNT;
		open($COUNT, "+> $pastno_file") || &error("Open Error: $pastno_file");
		print $COUNT "$N\n";
		truncate($COUNT, tell($COUNT));
		close($COUNT);

print "過去ログページカウントファイル: $[pastno_file]を作成しました。\n";

	}

	our $titlebb;
	# BBCode処理の変数関数を選択 (2009/01/14 スガラボット修正)
	if ($bbcode) { $titlebb ="bbtitle"; } 
	else { $titlebb = "titlebbrm"; }

	use File::Copy;

	my ($i, $k, $fcount, $tcount);
	local ($print_string, @upl_files);
	$k = $fcount = $tcount = 0;
	for ($i = 0; $i < $tp_count; $i++) { 

		# 現行ログに変化がある場合
		if ($nowfile_change && $i < $i_max) {

			# 新しい現行インデックス・ファイルに追加する
			push(@new_tp, $tp[$i]);

			# もし、過去ログから現行ログに繰り上がるスレッドがあればキーを変更する
			if ($i > $now_count) {
				my ($num) = split(/<>/, $tp[$i]);
				my $IN;
				open($IN, "+< $logdir/$num.cgi") || &error("Open Error: $num.cgi");
				my @log = &arrayDecode($IN, $logchar);
				my ($no, $lsub, $lres, $key, $old_key) = split(/<>/, $log[0]);
				if ($old_key > 0) { $key = $old_key; }
				else { $key = '1'; }
				$log[0] = "$no<>$lsub<>$lres<>$key<>\n";
				seek($IN, 0, 0);
				print $IN &arrayEncode(\@log, $logchar);
				truncate($IN, tell($IN));
				close($IN);
			}
		} else {

			# 過去ログ書庫ページがのスレッド数が一杯になったら次の書庫ページを作成する
			if ($k == $past_page_max_tno) {

				# 新しい過去ログインデックスファイルを作成する
				$n++;
				$N = sprintf("%04d", $n);
				$past_index[$n] = "$pastdir/P${N}.dat";
				open($INDEX_P, "+> $past_index[$n]");
#				@past_tp =&arrayDecode($INDEX_P);
    
				# 過去ログ書庫ディレクトリの作成
				if ($copy_logfiles) {
					$past_page_dir[$n] = "$pastdir/plog${N}";
					unless (-e $past_page_dir[$n]) {
						mkdir($past_page_dir[$n], '0707');
						mkdir("$past_page_dir[$n]/upl", '0707');
						$acount++;
					}
				}
    
				$k = 0;
			}

			# 新しい過去インデックス・ファイルに追加する
			push(@new_tp2, $tp[$i]);

			# 古いものから順にスレッドのツリー式インデックスを作成する
			$print_string = '';
			@upl_files = ();
			# Forum 表示リンクのパラメータセット
			$readshtml = "$cgi_url/read.cgi?bbs=forum&mode=past";
			# ツリー形式インデックス作成
			my $num = &create_pindex($tp[$element_no - $tcount - 1]);
			# Patio 表示リンクのパラメータセット
			$readshtml = "$cgi_url/read.cgi?bbs=patio&mode=past";
			unshift(@past_newtp, $tp[$tp_count - $tcount - 1].$print_string);
    
			# 過去ログに移動するスレッドのlog/datファイルとアップロードファイルをコピーする
			if ($copy_logfiles) {
				copy("$logdir/$num.cgi", "$past_page_dir[$n]/$num.cgi");	
				copy("$logdir/$num.dat", "$past_page_dir[$n]/$num.dat");	
#				print ("copy $logdir/$num.cgi $past_page_dir[$n]/<br>\n");
    
				while ( @upl_files ) {
					my $file = shift(@upl_files);
					copy("$upldir/$file",  "$past_page_dir[$n]/upl/$file");
#					print ("copy $upldir/$file $past_page_dir[$n]/upl/<br>\n");
    
					$fcount++;
				}
			}

			$tcount++;
			$k++;
			my $j = $i + 1;
			if ($k == $past_page_max_tno || $j == $tp_count) {

				# 新しい過去ログインデックスファイルを書き込む
				print $INDEX_P &arrayEncode(\@past_newtp, $logchar);
				truncate($INDEX_P, tell($INDEX_P));
				close($INDEX_P);
				@past_newtp = ();

print "過去ログインデックスファイル: $pastdir/P${N}.dat を書き込みました。\n\n";

				# 過去ログ書庫ページのカウントファイルを作成
				my $COUNT;
				open($COUNT, "+> $pastno_file") || &error("Open Error: $pastno_file");
				print $COUNT "$N<>$k<>\n";
				truncate($COUNT, tell($COUNT));
				close($COUNT);
			}
		}
	}

	# ログファイルに変更を加える場合は書き込み
	if ($nowfile_change) {
		# 現行ログ書込み
		seek($INDEX1, 0, 0);
		print $INDEX1 &arrayEncode(\@new_tp, $logchar);
		truncate($INDEX1, tell($INDEX1));
		close($INDEX1);

		# 過去ログ書込み
		seek($INDEX2, 0, 0);
		print $INDEX2 &arrayEncode(\@new_tp2, $logchar);
		truncate($INDEX2, tell($INDEX2));

print "現行ログインデックスと過去ログインデックスを書き込みました。\n";

	}

	# ログファイルをクローズ
	close($INDEX1);
	close($INDEX2);

	# 結果表示
	if ($exit_flag) {
		$printString .= <<"EOM";
<form action="$bbscgi">
<input type="submit" value="&lt; 掲示板">
</form>
EOM
	}

	$printString .= <<"EOM";
過去ログ書庫・再構築処理が終了しました。
作成した最終のページカウント:$N<>$k<>, 移動したスレッド数:$tcount, コピーしたファイル数:$fcount
EOM

print $printString;

}

# list display
#----------------------------------------------------
# ツリー構造のポインターリストを印刷するサブルーチン
#----------------------------------------------------

#	$pre = '';
#	$pre1a = '├';
#	$pre1b = '└';
#	$pre2a = '│';
#	$pre2b = '　';

sub lst_dsp3 {
    my ($idx, $pre) = @_;			# 引数はデータの行番号とプレフィックス文字列
    my ($i, $link);

#printf("lst_dsp2(%d)\n", $idx);

# デバッグプリント
#&error("check point No.4");

	$i = $idx;
	do {
		if ($next[$i] == $chid[$prnt[$i]]) {
			# 自分の次ノードが長男のポインタと同じなら最後のノード
			$pre1 = $pre.$pre1b;
			$pre2 = $pre.$pre2b;
		} else {
			# 最後以外のノード
			$pre1 = $pre.$pre1a;
			$pre2 = $pre.$pre2a;
		}
#		printf("%-20s %-11s %-10s\n",
#			$pre1.$data[$i], $date[$i], $user[$i]);

# もし特定のレスを指定したツリー表示ならそのレスを強調表示する
		if ($from > 0 && $from == $no[$i]) { $st1 = $strong1; $st2 = $strong2; }
		else { $st1 = ''; $st2 = ''; }

		# 名前からトリップ表示を削除するかどうか (きりしまさんの掲示板を参考にして修正、2009/04/07 スガラボット)
		if ($patio_trip) {
			$nam[$i] =~ s/[◆◇].+//g;
		}
		
# タイトルと名前に文字修飾機能追加 （2009/01/12 スガラボット修正)
		$sub[$i] = &$titlebb($sub[$i]);
		$nam[$i] = &$titlebb($nam[$i]);

		# 添付ファイルのアイコン定義
		$res_icon = '';
		foreach $k (1 .. 3) 
		{
			local($ex,$w,$h,$filename,$filetitle) = split(/,/, $upl[$k][$i]);
			if ($ex) { 
				$res_icon = "<img src=\"$imgurl/clipsls.gif\" border=\"0\"> ";
				push(@upl_files, "$tim[$i]-$k$ex");
			} else {
				last;
			}
		}

		# 最後の行番号ならツリー全体の降順表示リンクを追加
		undef $revdisp;
		undef $revtree;
		my $readshtml_p = $readshtml . "&pastdir=$pastdir/$past_page_dir[$n]";
		if ($i == ($nofline - 1)) {
			$revtree = "<a href=\"$readshtml&tree=1&no=$num&l=rev\"><img src=\"$imgurl/treerev2.gif\" border=\"0\"></a>";
			$revdisp = "<a href=\"$readshtml&tree=1&no=$num&l=new\"><img src=\"$imgurl/upbutton1.gif\" border=\"0\"></a>";
		}

		# 携帯電話からのアクセスのときはサブツリーのリンクを表示しない (2008/12/08 スガラボット修正)
		if (!$ua) {
			$print_string .= "<SPAN STYLE=\"font-family:'monospace';\"><font size=\"+1\" color=\"#338B33\">$pre1</font></SPAN>$st1" . 
			"<a href=\"$readshtml&tree=1&no=$num&l=$no[$i]-\"><font color=\"green\">▼</font></a>" .
				"<a href=\"$readshtml&tree=1&no=$num&l=-$no[$i]\"><font color=\"green\">△</font></a>" .
				" No.$no[$i] &nbsp;［$res_icon<a href=\"$readshtml&tree=1&no=$num&l=$no[$i]\">$sub[$i]</a>］$revtree$revdisp&nbsp;<b>$nam[$i]</b> &nbsp;$dat[$i] $st2<br>\n";
		} else {
			if (!$resto[$i]) { $head_mark = "●"; }
			else { $head_mark = "■"; }
			$print_string .= "<font size=\"+1\" color=\"#338B33\">$pre1</font><font color=\"green\" size=\"-2\"> $head_mark </font>$st1" .
				" No.$no[$i] &nbsp;［$res_icon<a href=\"$readshtml&tree=1&no=$num&l=$no[$i]\">$sub[$i]</a>］$revdisp&nbsp;<b>$nam[$i]</b> &nbsp;$dat[$i] $st2<br>\n";
		}
		
		if ($chid[$i] != 0) {
			# 自分に子供が有るとき
			if ($next[$i] == $chid[$prnt[$i]]) {
				# 最後のノード
			} else {
				# 最後以外のノード
			}
			&lst_dsp3($chid[$i], $pre2);		# 1レベル字下がりして再帰的に子ノードを印刷する
		}
	
		$i = $next[$i];						# 子供がないときは次ノードを印刷する
    } until ($i == $idx);
}

# create_pindex
#--------------------------------------------------------------------
# ツリー構造の過去ログ書庫ページ・インデックスを作成するサブルーチン
#--------------------------------------------------------------------
sub create_pindex {

	my ($buf) = @_;

	local ($num,$sub,$res,$nam,$date,$na2,$key,$upl,$ressub,$restime) = split(/<>/, $buf);

	# アラーム数定義
	my $alarm = int ( $m_max * 0.9 );

	# スレッド毎のツリー表示インデックス作成する
	# スレッドタイトルを抽出
	my $tsub = $sub;

	# スレッド読み込み
	my ($IN, @log);
	open($IN, "+< $logdir/$num.cgi") || &error("Open Error: $num.cgi");
	@log = &arrayDecode($IN, $logchar);
#	close($IN);

	my $top1 = $log[0];
	my ($no,$lsub,$lres,$key) = split(/<>/, $top1);

	# $key を過去ログの属性 "-1" に変更する
	$log[0] = "$no<>$lsub<>$lres<>-1<>$key<>\n";

	seek($IN, 0, 0);
	my @temp = @log;
	print $IN &arrayEncode(\@temp, $logchar);
	truncate($IN, tell($IN));
	close($IN);

	# ツリー構造のリンクリストを作成するためトピックスのログファイルを一度全部読み込む。
	local (@prnt, @chid, @next, @prev, $j);
	$prnt[0] = 0;
	$chid[0] = 0;
	$no[0] = 0;
	$j = 1;

	local (@no,@sub,@nam,@eml,@com,@dat,@ho,@pw,@url,@mvw,@myid,@tim,@upl,@idcrypt,@color,@resto);
	my $line;
	for ($line = 0; $log[$j]; $line++) {
		chomp($log[$j]);
		($no[$line],$sub[$line],$nam[$line],$eml[$line],$com[$line],$dat[$line],$ho[$line],$pw[$line],$url[$line],$mvw[$line],
				$myid[$line],$tim[$line],$upl[1][$line],$upl[2][$line],$upl[3][$line],$idcrypt[$line],$color[$line],$resto[$line])  = split(/<>/, $log[$j]);
		$prnt[$line] = $resto[$line];

		$j++;
	}
	local $nofline = $line;

# ツリー構造のインデックス・リストを作成
#	prnt: 親ノード (parent node)
#	chid: 子ノード (child node)
#	next: 次ノード (兄弟リンクのnext node)
#	prev: 前ノード (兄弟リンクのprevious node)

	# スレッドのツリー表示がずれるのを修正するため、ツリーリストを作成するときはその都度@lineno関数をクリアする (20081015 スガラボット修正)
	undef @lineno;

# デバッグプリント
#&error("check point No.4");

	for ($line = 0; $line < $nofline; $line++) {
		&lst_cre($line, $prnt[$line], $no[$line]);
	}

# デバッグプリント
#&error("check point No.4");

	if (!$init) {
		$print_string .=   "<tr><td>&nbsp;</td></tr>";
	}
	$init = 0;
	$print_string .=   "<Tr><td nowrap>\n";

	# アイコン定義
	if ($key eq '0' || $key eq '4') { $icon = 'fold3.gif'; }
	elsif ($key eq '3') { $icon = 'fold4.gif'; }
	elsif ($key eq '2') { $icon = 'look.gif'; }
	elsif ($res >= $alarm) { $icon = 'fold5.gif'; }
	elsif ($upl) { $icon = 'fold6.gif'; }
	else { $icon = 'fold1.gif'; }

	# 添付ファイルのアイコン定義
	$res_icon = '';
	my @upl_files;
	foreach $k (1 .. 3) 
	{
		local($ex,$w,$h,$filename,$filetitle) = split(/,/, $upl[$k][0]);
		if ($ex) { 
			$res_icon = "<img src=\"$imgurl/clipsls.gif\" border=\"0\"> ";
			push(@upl_files, "$tim[0]-$k$ex");
		} else {
			last;
		}
	}

# スレッ覧からトリップを消したい場合有効にする
#	$nam[0] =~ s/◆.+//g;
#	$na2[0] =~ s/◆.+//g;

	# 名前からトリップ表示を削除するかどうか (きりしまさんの掲示板を参考にして修正、2009/04/07 スガラボット)
	if ($patio_trip) {
		$nam[0] =~ s/[◆◇].+//g;
	}
	
	# タイトルと名前に文字修飾機能追加 （2009/01/12 スガラボット修正)
	$sub[0] = &$titlebb($sub[0]);	
	$nam[0] = &$titlebb($nam[0]);	
	$tsub = &$titlebb($tsub);

	#スレッドタイトルとレス親題名を分離したことに因る処理
	local ($sub1, $sub2);
	if ($sub[0] ne $tsub) {
		$sub_dif = 1;
		$sub1 = "&nbsp;&nbsp;<b>スレッド $num: $tsub &nbsp;</b><br><img src=\"$imgurl/spacer_h4.gif\"><br>";
		$sub2 = " No.0 &nbsp;";
	} else {
		$sub2 = "<b>スレッド $num:</b>";
	}

	# スレッドの親記事を表示
	$print_string .=  $sub1;

	my $readshtml_p = $readshtml . "&pastdir=$pastdir/$past_page_dir[$n]";
	# 携帯からのアクセスのときは全表示でツリー表示をしない （2008/12/08 スガラボット修正)
	if (!$ua) {
		$print_string .=  "&nbsp;&nbsp;&nbsp;<a href=\"$readshtml&tree=1&no=$num&l=all\"><img src=\"$imgurl/tree.gif\" border=\"0\"></a>\n";
		$print_string .=  " <a href=\"$readshtml&no=$num\"><img src=\"$imgurl/$icon\" border=\"0\"></a> $st1 $sub2 ［$res_icon<a href=\"$readshtml&tree=1&no=$num&l=0\">$sub[0]</a>］&nbsp;<b>$nam[0]</b> &nbsp;$dat[0] $st2 \n";
	} else {
		$print_string .=  "&nbsp;&nbsp;&nbsp;<a href=\"$readshtml&no=$num\"><img src=\"$imgurl/$icon\" border=\"0\"></a>\n";
		$print_string .=  " $sub2 ［$res_icon<a href=\"$readshtml&tree=1&no=$num&l=0\">$sub[0]</a>］&nbsp;<b>$nam[0]</b> &nbsp;$dat[0] $st2 \n";
	}

	$print_string .= "&nbsp;$dat[0]<br>\n";


#	$pre = '　';
#	$pre1a = '├';
#	$pre1b = '└';
#	$pre2a = '│';
#	$pre2b = '　';
	
	$pre = "<img src=\"$imgurl/blank.gif\">";
	$pre1a = "<img src=\"$imgurl/branch.gif\">";
	$pre1b = "<img src=\"$imgurl/next.gif\">";
	$pre2a = "<img src=\"$imgurl/down.gif\">";
	$pre2b = "<img src=\"$imgurl/blank.gif\">";

# デバッグプリント
#&error("check point No.4");

	# スレッドを階層表示
	if ($chid[0]) {
		&lst_dsp3($chid[0], $pre);
	}
	$print_string .=   "</td></tr>\n";
#	$print_string .=   "</table><br>\n";

# デバッグプリント
#&error("check point No.4");

print "No.$num スレッドのインデックスを作成しました。\n";

#	push(@num_list, $num);
	return $num;
}

1;
