#!/usr/local/bin/perl
# ---------------------------------------------------
# CGIファイル名: sjis2utf8-5c.cgi
# 作成・著作権者: スガラボット
# ご利用条件: 個人的なご利用・及び再配布は自由です。但し商業利用されるときは下のホームURLのメールフォームから作者までご連絡ください。 
# ホームURL: http://vmorita.com/
# 掲示板URL: http://vmorita.com/bbs/patio/patio.shtml
# ---------------------------------------------------
#
# ・sjis2utf8-5c.cgiは指定されたディレクトリまたはカレントディレクトリ配下の指定ファイルの文字コードをShift_JIS(CP932)からUTF-8へ一括変換します。
# 　ファィルがperlCGIファイルの時など、内部の文字リテラルで"表示"などのように5Cエスケープされた表現があれば一括して"\"の文字を削除します。
# コマンド形式は以下の通りです。
# ・perl fileConvert.cgi [-dir=DIR1] [-dir=DIR2] … [-dir=DIRN] [file1] [file2] … [fileN] [-undo] (リターン)
# ・[-dir=DIR1] [-dir=DIR2] … [-dir=DIRN] オプションは、"～"で処理対象のディレクトリ (絶対パスまたは相対パス) を指定します。複数のディレクトリが指定できます。
# ・"-dir=～"が省略されたときはカレントディレクトリが対象になります。
# ・デフォールトではこの処理は入力ファイルの文字列が shiftjis(CP932を含みます) 等指定のコードと判定できないときは変換を中止します。
# ・[file1] [file2] … [fileN] で個別ファイルを指定するほか '*.cgi' や '??.cgi' のように"*"と"?"をワイルドカードとして使用できます。
# ・(注: ワイルドカードで指定するとき、LinuxなどUnix系のOSの時は必ず、上のように''で囲むか、\*.cgi のように「*」や「?」をエスケープしてください。Windowsでは必要ありません。)
# ・変換元のファイルは ターゲットディレクトリの下に新たに作成する "sjis_bak" というフォルダーにバックアップされます。 
# ・ファイルのコードを変換した後、前と全く同じオプションを指定して最後に "-undo" オプションを付けると前に行った変換をすべて元に戻す事が出来ます。
# ・最初のターゲットディレクトリ内に追記型のログ"sjis2utf8_log.txt"を出力します。このログファイルにより実行した処理内容を確認することができます。
# ・全く何もパラメータが指定されていないときは、sjis2utf8-5c.cgi は何も処理しません。

use strict;
use warnings;

use File::Basename;
my $script = basename($0);

use utf8;

our $termCharset;
if ($ENV{'WINDIR'}) { $termCharset = 'cp932'; }
else { (undef,$termCharset) = split(/\./, $ENV{'LANG'}); }

eval "use Encode;";
if ($@) { print "This server cannot execute this script. Please use perl version 5.8.1 or greater!!\n"; exit; }

use Encode::Guess qw( euc-jp cp932 7bit-jis utf8 );

binmode STDOUT, ":encoding($termCharset)";
binmode STDERR,  ":encoding($termCharset)";

my @targetDir;
our @targetFile;
our $undo = 0;
our $backDir = 'sjis_bak';
our $br = '';
our $LOG;
our $pre = '';

if ($script eq 'sjis2utf8-5c.cgi') {
	use strict;
	use warnings;

	foreach (@ARGV) {
		if ($_ =~ /^-dir=(.+)/) { push (@targetDir, $1); next; }
		if ($_ eq '-undo') { $undo = 1; next; }
		push (@targetFile, decode($termCharset, $_));
	}
	if (!@targetDir) { push(@targetDir, '.'); }

	# ログファイルをオープン
	my $encMethod = ($termCharset =~ /utf-?8/i) ? ':utf8' : ":encoding($termCharset)";
	open($LOG, ">>$encMethod", "$targetDir[0]/sjis2utf8_log.txt") or die("ログファイル $targetDir[0]/sjis2utf8_log.txt が開けません\n");

	my (undef, $date) = &get_time;
	print $LOG "######### これからファィルの文字コード変換・5C文字エスケープの消去処理を開始します ($date) #########\n";
	print $LOG "COMMAND = $script ";
	foreach(@ARGV) {
		my $str = decode($termCharset, $_);
		print $LOG "$str ";
	}
	print $LOG "\n";

	# 各ディレクトリ内のファイル変換処理
	foreach my $dirName (@targetDir) {
		&del5cDir($dirName);
	}

	(undef, $date) = &get_time;
	print $LOG "--------- ファィルの文字コード変換・5C文字エスケープの消去処理を終了しました ($date) ---------\n\n\n";

	close($LOG);
}

sub del5cDir {
	my ($dirName) = @_;

	if ($undo) { &printLog("********* ${dirName} ディレクトリのファイル文字コード変換と5C文字エスケープ削除を元に戻します *********${br}\n"); }
	else {&printLog("********* ${dirName} ディレクトリのファイル文字コード変換と5C文字エスケープ削除を開始します *********${br}\n"); }

	my $DIR;
	unless (opendir($DIR, $dirName)) { printLog("${dirName}の情報を取得できません\n"); return; }
	my (@file) = readdir($DIR);
	closedir($DIR);

	my $count = 0;
	my $fcount = 0;
	foreach my $f (@file) {
		my $f_org = $f;
		$f = decode($termCharset, $f);
		&printLog("${pre}file= " . $f . " ");
		my ($fb, $ext) = split(/\./, $f, 2);
		if ($f eq '.' || $f eq '..') { &printLog("スキップ${br}\n"); next; }
		if ($f  =~ /^sjis2utf8-5c\w*\.cgi/) { &printLog("スキップ<br>\n"); next; }
		if (opendir(my $FH, "${dirName}/$f_org")) { &printLog("これはディレクトリです${br}\n"); closedir($FH); next; }

		my $ok = 0;
		foreach my $tfile (@targetFile) {
			if ($f eq $tfile) { $ok = 1; last; }
			my $temp = $tfile;
			$temp =~ s/\*/\\S*/;
			$temp =~ s/\?/\\S/;
			$temp =~ s/\./\\./;
			if ($f =~ /^$temp$/) {
				if ($f =~ /bak/ && $temp !~ /bak/) { next; }
				$ok = 1;
				last;
			}
			next;
		}
		if (!$ok) { &printLog("スキップ${br}\n"); next; }
	
		# バックファイルがあるとき、undo処理ならバックファイルを元のファイルにリネーム、変換処理ならスキップ
		if (-e "${dirName}/${backDir}/${f_org}") {
			if ($undo) {
				rename("${dirName}/${backDir}/${f_org}", "${dirName}/$f_org");
				&printLog("ファイルのコード変換を元に戻しました${br}\n");
				$fcount++;
				next;
			} else {
				&printLog("同処理のバックファイルがあるためスキップ${br}\n");
				next;
			}
		}

		# もし undo 処理なら以降をスキップ
		if ($undo) { &printLog("スキップ${br}\n"); next; }

		# もしバックアップディレクトリがなければ作成する
		unless (-e "${dirName}/${backDir}") { mkdir("${dirName}/${backDir}", 0755); }

		# 元のファイルをバックアップディレクトリにムーブする
		rename("${dirName}/${f_org}", "${dirName}/${backDir}/${f_org}");
		my $IN;
		my $OUT;

		unless (open $IN, '<', "${dirName}/${backDir}/${f_org}") { &printLog("のバックアップファイルが読み込めませんでした<br>\n"); next; }

		# パーミッション取得
		my $permission = (stat($IN))[2] % 512;

		# データ読み込み
		my $data;
		$data .= $_ while (<$IN>);
		close($IN);

		# 入力文字コードを判定 (指定コード以外のファイルは変換しない)
		my $enc = guess_encoding($data);
		my $encName = $enc->name;
		if ($encName ne 'cp932') {
			rename("${dirName}/${backDir}/${f_org}", "${dirName}/${f_org}");
			&printLog("の文字コード \"は$encName\" で \'cp932(sjis)\' ではないため変換を中止しました<br>\n");
			next;
		}

		# 再度入力ファイルを読み込み
		unless (open $IN, '<:encoding(cp932)', "${dirName}/${backDir}/${f_org}") { &printLog("のバックファイルが読み込めませんでした${br}\n"); next; }
		
		unless (open $OUT, '>:utf8', "${dirName}/${f_org}") {
			rename("${dirName}/${backDir}/${f_org}", "${dirName}/${f_org}");
			&printLog("のファイルがオープンできませんでした${br}\n");
			next;
		}

		# 処理したファイル数をインクリメント
		$fcount++;

		&printLog("の文字コードを変換しました。これから5Cエスケープ文字の削除を実行します。${br}\n");
		my $line = 0;
		my $no = 0;
		my $sum = 0;
		$pre = '　　';
		while (<$IN>) {
			$data = $_;
			$line++;
			($data, $no) = &del5cEscape($data, $line, $pre);
			$sum += $no;
			print $OUT $data;
		}

		&printLog("${pre}このファイルで総数 ${sum} 個の5Cエスケープ文字を削除しました${br}\n");
		$count += $sum;

		close($OUT);
		chmod($permission, "${dirName}/${f_org}");
		close($IN);
		$pre = '';
	}

	if ($undo) {
		&printLog("********* ${dirName} ディレクトリで ${fcount} ファイルのコード変換と5C文字エスケープ処理を元に戻しました。 *********${br}\n");
	} else {
		&printLog("********* ${dirName} ディレクトリで ${fcount} ファイルのコード変換をし、${count} 個の5Cエスケープ文字を削除しました *********${br}\n");
	}
}

sub del5cEscape {
	my ($data, $line, $pre) = @_;
	my $count = 0;
	
	my @sjis5c = ('―', 'ソ', 'Ы', 'Ⅸ', '噂', '浬', '欺', '圭', '構', '蚕', '十', '申', '曾', '箪', '貼', '能',
					'表', '暴', '予', '禄', '兔', '喀', '媾', '彌', '拿', '杤', '歃', '濬', '畚', '秉', '綵', '臀',
					'藹', '觸', '軆', '鐔', '饅', '鷭', '偆', '砡', '纊', '犾');

	my $i =0;
	foreach my $char (@sjis5c) {
		while ($data =~ /$char\\/) {
			$data =~ s/($char)\\/$1/;
			&printLog("${pre}${line}行目: ${char}\\ → ${char} に変換${br}\n");
			$count++;
		}
		$i++;
	}
	if ($count) { &printLog("${pre}${line}行目で $count 個変換しました${br}\n"); }
	return ($data, $count);
}

sub printLog {
	my ($data) = @_;
	print $LOG ($data);
	print $data;
}

sub get_time {
	$ENV{'TZ'} = "JST-9";
	my($time) = time;
	my($sec,$min,$hour,$mday,$mon,$year) = (localtime($time))[0..5];

	# 日時のフォーマット
	my($date) = sprintf("%04d/%02d/%02d %02d:%02d:%02d",
				$year+1900,$mon+1,$mday,$hour,$min,$sec);
	return ($time, $date);
}

1;
