#!/usr/local/bin/perl
# ---------------------------------------------------
# CGIファイル名: wforum_logConvert.cgi
# 作成・著作権者: スガラボット
# ご利用条件: 個人的なご利用・及び再配布は自由です。但し商業利用されるとき、再配布されるときは下のホームURLのメールフォームから作者までご連絡ください。 
# ホームURL: http://vmorita.com/
# 掲示板URL: http://vmorita.com/bbs/patio/patio.shtml
# ---------------------------------------------------
#
# 本CGIスクリプトはスガラボットの Forum Patio v1.35 以降の掲示板CGIに、既存のWeb Forumログを適合させるための変換プログラムです。
# 利用可能なWeb Forum 掲示板は、KENT式およびこれに画像添付機能等を拡張した「掲示板改造支援サイト」で配布されているWeb Forum (多機能版: URLは以下) です。
# http://swanbay-web.hp.infoseek.co.jp/wforum_ex/index.html
#
# 利用方法は、Web Forumがインストールされたサーバの別ディレクトリにForum Patioをインストールして、cgi-bin内の本CGIをPCからブラウザでアクセスするだけ完了です。
# ディレクトリの配置は、以下のようにwforum がご利用ホームページのトップディレクトリにインストールされている場合、forum_patio_v1xxm などを同じようにトップディレクトリ
# にインストールしてご利用下さい。
# 例えばさくらレンタルサーバにForum Patio v1.36m を forum_patio_v135m というサブディレクトリにインストールす場合、以下のようになります。
#
# /home/USER-ID/www/wforum/admin.cgi
#                          index.cgi
#                             :
#                          data/log.cgi
#                                 :
#                          img/xxxx.jpg
#                                 :
#                  /forum_patio_v136s/cgi-bin/wforum_logConvert.cgi
#                                             init.cgi
#                                             forum.cgi
#                                                :
#                                             data/index1.log
#                                                     :
#                                             log/xx.cgi
#                                                 xx.dat
#                                                     :
#                                             upl/xxxxxxxxxx.jpg
#                                                     :
#                                     bbs/room1/forum.shtml  (これ以下の行は複数会議室版の例)
#                                                     :
#                                               data/index1.log
#                                                     :
#                                               log/xxx.cgi
#                                                   xxx.dat
#                                                     :
#                                               upl/xxxxxxxxxx.jpet
#                                                     :
#                                         room2/forum.shtml
#                                                     :
#
# 以上の例では本CGiはデフォールトのまま使用できます。ディレクトリ配置がこれと子なる場合は適宜以下の設定項目を変更してご利用下さい。
# この時、PCブラウザのURL欄には以下のように入力します
#
# http://sakura.ne.jp/USER-ID/forum_patio_v135m/cgi-bin/wforum_logConvert.cgi?room=room1
#
# 上記で ?room=xxxx の会議室名の値はご利用状況に合わせてご使用の会議室名を入力して下さい。
#
# 単一会議室版を上記の複数会議室版と同様 forum_patio_v135s というサブディレクトリにインストールしたときは、37行目以下が無いのと、?room=xxxx は不要です。
#
# 本CGIをコマンドラインから使用することも出来ます。
# サーバのディレクトリ配置が上の例と同じ場合は、コンソールから wforum_logConvert.cgi を直接起動して下さい。このとき以下のようにパラメータが設定できます。
# [usr@server www]$ perl wforum_logConvert.cgi -d=/home/USER-ID/www/wforum -to=utf8
# 上記の例では、
# -d=～ の ～ で Web Forum のCGI ディレクトリを指定できます。
# -to=utf8 の utf8 の代わりに euc (EUC-JP) や sjis (ShiftJis)を指定することが出来ます。
# これらのパラメータを指定しないと以下の設定項目で指定されたデフォールト値が適用されます。
#
# 過去ログの変換は現状ではサポートされていません。127～128行目の $targetFile と $targetImdDir を適宜書き換えれば可能と思いますのでチャレンジしてみて下さい。

#-------------------------------------------------
#  wforum ログ変換処理の設定項目
#-------------------------------------------------
# ログ変換する wforum CGI のインストールディレクトリ (絶対または相対パス)
our $targetDir = "../../wforum";

# 変換前 wforum ログの文字コード (デフォールトは sjis)
our $code1 = 'sjis';

# 変換後のログの文字コード (デフォールトはinit.cgiの $logchar)

# 変換したログを現在の Forum Patio のログに追加する (=1: 追加する)、(=0: 現在のログを削除して新規に作成する)
our $log_add = 1;

# 変換前の記事番号をタイトルの後に [No.XX]として残すかどうか (=1: 残す)
our $oldNo = 1;
# ---------------------------------------------------

#use strict;
#use warnings;
use utf8;

use lib './lib';
use Unicode::Japanese;
use Unicode::Japanese qw(unijp PurePerl);

require 'init.cgi';

our $termCharset;
our $code2 = $logchar;

if ($ENV{'WINDIR'}) { $termCharset = 'sjis'; }
else { (undef,$termCharset) = split(/\./, $ENV{'LANG'}); }

our $httpRequest = 0;
our $br;
our $end;
our ($room, $roomdir, %in, $def_room, $sub_dir, $root_dir, $bbs_dir, $targetFile);

if ($ENV{'REQUEST_METHOD'}) {
	$httpRequest = 1;
#	$termCharset = 'utf8';
	$br = '<br>';
	$end = '<br></body></html>';
}

#eval "use Encode;";
#if ($@) { print "This server cannot execute this script. Please use perl version 5.8.1 or greater!!\n"; exit; }
#use Encode::Guess qw( euc-jp cp932 7bit-jis utf8 );

#binmode STDOUT, ":encoding($termCharset)";
#binmode STDERR,  ":encoding($termCharset)";

if ($httpRequest) {

	$termCharset = $charset;
	our $htmlcode = &htmlchar($termCharset);

	print "Content-Type: text/html; charset=$htmlchar\nPragma: no-cache\nCache-Control: no-cache\n\n";

	my $printString = <<"EOM";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html lang="ja">
<head>
<meta http-equiv="content-type" content="text/html; charset=$htmlchar">
<title>wforumログファイルをForum Patioログファイルに変換</title>
</head>
<body>
EOM

	print &strEncode($printString, $termCharset);

#	nouse strict;

} else {
	foreach (@ARGV) {
	#	print $_ . "\n";
		if ($_ =~ /^-d=(.+)/) { $targetDir = $1; next; }
		if ($_ =~ /^-to=([\w\-]+)/) { $code2 = &codeCheck($1) || die("文字コードの指定が違います${br}\n"); next; }
	}
}

our $targetFile = "$targetDir/data/log.cgi";
our $targetImgDir = "$targetDir/img";

print &strEncode("処理パラメータ: room=$room, target=$targetFile, code2=$code2${br}\n", $termCharset);

# ファイル変換処理実行
&logConvert($targetFile);

print &strEncode("######### ファイルの文字コード変換処理を終了しました #########\n", $termCharset);

if ($httpRequest) {
	$printString = <<"EOM";
<br><br>
<a href="$forumcgi">変換した掲示板トップページを表示</a>
$end
EOM

print &strEncode($printString, $termCharset);
}

exit;

#-------------------------------------------------
#  ログ変換のメインサブルーチン
#-------------------------------------------------
sub logConvert {

	my ($targetFile) = @_;

	print &strEncode("${pre}********* $targetFile の変換を開始します *********${br}\n", $termCharset);

	local ($IDX, $id_top, @IDATA, $host0, $recentPost0);

	local $max_thread = 0;
	# ログファイルを既存のログに追加の場合
	if ($log_add) {

		# インデックスファイルをオープン
		if (open($IDX, "+< $nowfile")) {

			# 現行のインデックスデータ読み込み
			@IDATA = &arrayDecode($IDX);

			# 現行のインデックスファイルをバックアップ
			my $BAK;
			open($BAK, "> $nowfile.bak");
			print $BAK &arrayEncode(\@IDATA);
			close($BAK);

			$id_top = shift(@IDATA);
			($max_thread, $host0, $recentPost0) = split(/<>/, $id_top);

			# インデックスファイルを巻き戻しておく
			seek($IDX, 0, 0);
		}
		# ログ追加モード指定なのに既存のインデックスファイルが無い場合は新規作成する
		else {
			open($IDX, "> $nowfile") || die ("$nowfile が開けません!!");
		}
	}
	# ログ新規作成の場合
	else {
		# もし既存のindex.logファイルがあればバックアップ
		if (-e "$nowfile") { rename($nowfile, "$nowfile.bak"); }

		# インデックスファイル新規作成
		open($IDX, "> $nowfile") || die ("$nowfile が開けません!!");
	}

	# indexファイルの各項目変数を定義
	local ($tline, @tnum, @ttitle, @tkijino, @tnam, @tdate, @tlastnam, @tlastsub, @ttime, @maxtim, $tmaxtime);

	local $FROM;
	open($FROM, "< $targetFile") || die ("$targetFile が読み込めません!!");

	local (@log) = &arrayDecode($FROM, $code1);

	$top = shift(@log);

	local ($maxNum, $host, $recentPost) = split('<>', $top);

	local (@thread, @kiji_order, @TLOG, @TLOG00, @sub, @nam, @resnum, @lnum, @fname, $w, $h);
	local (@num, @prnt, @indent, @title, @email, @url, @name, @date, @com, @tim, 
			@host, @pkey, @wrap, @oya, @smail, @res, @color, @ex, @w, @h, @id, @uni);
	
	# パラメータ初期化
	local $threadno = $max_thread;
	local $kijino;
	local $line = 0;
	foreach (@log) {
		chomp;
		# 各行のパラメータを格納
		local ($num[$line], $prnt[$line], $indent[$line], $title[$line], $email[$line], $url[$line], $name[$line], 
			$date[$line], $com[$line], $time[$line], $host[$line], $pkey[$line], $wrap[$line], $oya[$line], $smail[$line], 
			$res[$line], $color[$line], $ex[$line], undef, undef, $id[$line], $uni[$line]) = split(/<>/);

		# 親記事を検出すると行う処理
		if ($prnt[$line] eq 'no') {
			# 前スレッドの処理中ならこれまでのスレッドをログとして書き出し
			if($threadno > $max_thread) {
				&thread_write;
			}
			# スレッド処理の初期化
			$threadno++;
			$kijino = 0;
			$tmaxtime = 0;
			@kiji_order = ();
			@maxtim = ();
			@lnum = ();
			@TLOG = ();
			@TLOG00 = ();
			$tnum[$tline] = $threadno;
			$ttitle[$tline] = $title[$line];
			$tnam[$tline] = $name[$line];
			$tdate[$tline] = $date[$line];
		}

		# この行記事のスレッド番号
		$thread[$line] = $threadno;

		# この行記事のスレッド内順位を記録
		$kiji_order[$kijino] = "$num[$line]<>$kijino<>\n";

		# スレッド内記事のタイムスタンプを記録
		$maxtim[$kijino] = $time[$line];

		# 記事タイトルと投稿者をスレッド内番号で記録
		$sub[$kijino] = $title[$line];
		$nam[$kijino] = $name[$line];

		# 添付ファイルのコピー処理
		if ($ex[$line]) { 
			my $BFILE;
			open($BFILE, "< $targetImgDir/$num[$line]$ex[$line]") || &error("$targetImgDir/$num[$line]$ext[$line] ファイルが開けません!!");
			binmode ( $BFILE );
			my ( $fsize ) = -s $BFILE;

			# 添付ファイル読み込み
			my $buf;
			read ( $BFILE, $buf, $fsize );
			close($BFILE);

			# コピー先ファイルをオープン
			$fname[$line] = "$time[$line]-1$ex[$line]";
			my $copyfile = "$upldir/$fname[$line]";

			# 書き込み
			my $OUT;
			open($OUT,">$copyfile") || &error("$imgfileオープンエラー、ファイルコピーを失敗しました!!");
			binmode($OUT);
			syswrite ( $OUT, $buf, $fsize );
			close($OUT);

			# 画像ファイルの場合はサイズ取得
			if ($ex[$line] eq ".jpg") { ($w[$line], $h[$line]) = &j_size($copyfile); }
			elsif ($ex[$line] eq ".gif") { ($w[$line], $h[$line]) = &g_size($copyfile); }
			elsif ($ex[$line] eq ".png") { ($w[$line], $h[$line]) = &p_size($copyfile); }
		}

		# 記事情報の位置行を新ログの形式で記憶
		$TLOG00[$kijino] = "$name[$line]<>$email[$line]<>$com[$line]<>$date[$line]<>$host[$line]<>$pkey[$line]<>$url[$line]<>$smail[$line]<>";
		$TLOG00[$kijino] .= "$id[$line]<>$time[$line]<>$ex[$line],$w[$line],$h[$line],$fname[$line],$fname[$line]<>,,,,<>,,,,<>$id[$line]<>$color[$line]<>";
		$resnum[$kijino] = $prnt[$line];

		$kijino++;
		$line++;
	}

	# 最後のスレッド書き出し処理
	&thread_write;

	# 既存ログに追加モードで書き込む場合は､既存ログの最新アクセスと Web Forum ログの最新アクセスを比較
	if ($log_add) {
		if ($recentPost0 > $recentPost) {
			$recentpost = $recentPost0;
			$host = $host0;
		}
	}

	# index ファイル第1行目に掲示板の最新アクセス状態を記入
	unshift(@IDATA, "$threadno<>$host<>$recentPost<>\n");

	# index ファイル書きだし
	print $IDX &arrayEncode(\@IDATA, $code2);
	truncate($IDX, tell($IDX));
	close($IDX);

	return;
}

#-------------------------------------------------
#  スレッドのログを抽出保存
#-------------------------------------------------
sub thread_write {

	# もし既に同名のログファイルがあればバックアップ
	if (-e "$logdir/$threadno.cgi") {
		# もしバックアップディレクトリがなければ作成する
		unless (-e "$logdir/bak") { mkdir("$logdir/bak", 0777); }
		rename("$logdir/$threadno.cgi", "$logdir/bak/$threadno.cgi");
		rename("$logdir/$threadno.dat", "$logdir/bak/$threadno.dat");
	}

	# $LOGファイル作成
	my $LOG;
	open($LOG, "> $logdir/$threadno.cgi") || die ("$threadno.cgi のlogfile が開けません!!");

	# @TLOGファイルをnum順にソート
	my @kiji_sort = sort { $a <=> $b } @kiji_order;

	local $i = 0;
	foreach (@kiji_sort) {
		my ($anum, $n) = split(/<>/);
		$tmaxtime = $maxtim[$n];
		$lnum{$anum} = $i;
		$tlastnam[$tline] = $nam[$n];
		$tlastsub[$tline] = $sub[$n];
		my $oldnum = $oldNo ? " [No.$anum]" : "";
		$TLOG[$i] = "$i<>$sub[$n]${oldnum}<>$TLOG00[$n]";
		if (!$i) { $TLOG[$i] .= "<>\n"; }
		else {	   $TLOG[$i] .= "$lnum{$resnum[$n]}<>\n"; }
		$i++;
	}

	$kijino--;
	$tkijino[$tline] = $kijino;
	$ttime[$tline] = $tmaxtime;

	# スレッドの最新状態をファイル第一行に書込み
	unshift(@TLOG, "$threadno<>$ttitle[$tline]<>$tkijino[$tline]<>1<>\n");

	# @TLOG_sort をログファイルに書き込み
#	print $LOG &arrayEncode(\@kiji_order, $logchar);
#	print $LOG "\n";
#	print $LOG &arrayEncode(\@kiji_sort, $logchar);
#	print $LOG "\n";
	print $LOG &arrayEncode(\@TLOG, $code2);
	truncate($LOG, tell($LOG));
	close($LOG);
	
	# 参照ファイル生成
	my $NO;
	open($NO,">$logdir/$threadno.dat") || &error("Write Error: threadno.dat");
	print $NO "0:";
	close($NO);

	# パーミッション変更
	chmod(0666, "$logdir/$threadno.cgi");
	chmod(0666, "$logdir/$threadno.dat");

	# index に１行書き出し
	push(@IDATA, "$tnum[$tline]<>$ttitle[$tline]<>$tkijino[$tline]<>$tnam[$tline]<>$tdate[$tline]<>$tlastnam[$tline]<>1<><>$tlastsub[$tline]<>$ttime[$tline]<>\n");
	$tline++;
}

#-------------------------------------------------
# コードチェック
#-------------------------------------------------
sub codeCheck {
	($_) = @_;
	if (/shift.*jis/i || /sjis/i) { return 'sjis'; }
	elsif (/euc.*jp/i || /jp.*euc/i || /ujis/i) { return 'euc-jp'; }
	elsif (/jis/i) { return '7bit-jis'; }
	elsif (/utf-?8/i) { return 'utf8'; }
	else { return ''; }
}

#-------------------------------------------------
#  JPEGサイズ認識
#-------------------------------------------------
sub j_size {
	local($jpeg) = @_;
	local($t, $m, $c, $l, $W, $H);

	open(JPEG, "$jpeg") || return (0,0);
	binmode JPEG;
	read(JPEG, $t, 2);
	while (1) {
		read(JPEG, $t, 4);
		($m, $c, $l) = unpack("a a n", $t);

		if ($m ne "\xFF") {
			$W = $H = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(JPEG, $t, 5);
			($H, $W) = unpack("xnn", $t);
			last;
		} else {
			read(JPEG, $t, ($l - 2));
		}
	}
	close(JPEG);
	return ($W, $H);
}

#-------------------------------------------------
#  GIFサイズ認識
#-------------------------------------------------
sub g_size {
	local($gif) = @_;
	local($data);

	open(GIF,"$gif") || return (0,0);
	binmode(GIF);
	sysread(GIF,$data,10);
	close(GIF);

	if ($data =~ /^GIF/) { $data = substr($data,-4); }

	$W = unpack("v",substr($data,0,2));
	$H = unpack("v",substr($data,2,2));
	return ($W, $H);
}

#-------------------------------------------------
#  PNGサイズ認識
#-------------------------------------------------
sub p_size {
	local($png) = @_;
	local($data);

	open(PNG, "$png") || return (0,0);
	binmode(PNG);
	read(PNG, $data, 24);
	close(PNG);

	$W = unpack("N", substr($data, 16, 20));
	$H = unpack("N", substr($data, 20, 24));
	return ($W, $H);
}

